#  SPDX-FileCopyrightText: 2023 Atmark Techno contributors
#
#  SPDX-License-Identifier: MIT

# pyright: reportPrivateImportUsage=false

import logging
from re import compile

from license_expression import get_spdx_licensing, ExpressionError
from spdx_tools.spdx.model import (
    Package,
    SpdxNoAssertion,
    Relationship,
    RelationshipType,
)
from typing import List

from utility import get_spdx_id


class PackagesSbom:
    def __init__(self, filepath: str) -> None:
        self.packages = self.get_packages_sbom(filepath)

    def get_relationships(self, spdx_id: str) -> List[Relationship]:
        return [
            Relationship(
                spdx_element_id=spdx_id,
                relationship_type=RelationshipType.CONTAINS,
                related_spdx_element_id=package.spdx_id,
            )
            for package in self.packages
        ]

    def packages_parse(self, filepath: str) -> tuple[list[str], list[str], list[str]]:
        with open(filepath) as file:
            # Store in an array with newline delimiters
            package_list = file.readlines()

        # Stored in a space-separated two-dimensional array
        # Data is stored as follows
        # ['abos-base-2.0-r1', 'aarch64', '{abos-base}', '(MIT)', '[installed]\n']
        extract_re = compile(r"([^ ]*)-([^-]*-r[^ ]*) [^(]*\(([^)]*)\)")
        packages_match = [extract_re.match(package) for package in package_list]

        names, versions, licenses = [], [], []
        for package in packages_match:
            # Ensure that the package does not have the type None
            if package is None:
                continue
            name, version, license = package.groups()
            names.append(name)
            versions.append(version)
            licenses.append(license)

        return names, versions, licenses

    def get_packages_sbom(self, filepath: str) -> List[Package]:
        names, versions, licenses = self.packages_parse(filepath)
        packages = []

        for name, version, license in zip(names, versions, licenses):
            # Receive ExpressionError if license cannot be obtained
            try:
                license_concluded = get_spdx_licensing().parse(license, validate=True)
            except ExpressionError:
                license_concluded = SpdxNoAssertion()
                logging.warning(f"Failed to parse {name} license: {license}")
            packages += [
                Package(
                    spdx_id=get_spdx_id(name),
                    name=name,
                    download_location=SpdxNoAssertion(),
                    version=version,
                    license_concluded=license_concluded,
                    license_declared=SpdxNoAssertion(),
                    copyright_text=SpdxNoAssertion(),
                    files_analyzed=False,
                )
            ]
        return packages
